/* esdb.h: Evans & Sutherland Double-buffering Escape */
/*
 * These 2 macros will eventually be defined in PEX.h.
 */
#ifndef PEXEncodeEscape
#define PEXEncodeEscape(vendor, id)	(0x80000000 | (vendor<<16) | id)
#endif
#ifndef PEXEncodeEnum
#define PEXEncodeEnum(vendor, id)	(0x8000 | (vendor<<8) | id)
#endif

#define ES_VENDOR_ID			4
#define HP_VENDOR_ID			7

#define ESEscapeDoubleBufferID		1
#define ESEscapeSwapBufferID		2
#define ESEscapeSwapBufferContentID	3
#define HPEscapeDFrontID		1

#define ESETMEscapeDoubleBuffer		"ES_ESCAPE_DBLBUFFER"
#define ESETMEscapeSwapBuffer		"ES_ESCAPE_SWAPBUFFER"
#define ESETMEscapeSwapBufferContent	"ES_ESCAPE_SWAPBUFFERCONTENT"
#define HPETMEscapeDFront		"HP_ESCAPE_DFRONT"

#define ES_RENDERER_SINGLEBUFFER	0
#define ES_RENDERER_DBLBUFFER		1

#define ES_DB_SWAP_CONTENT_UNDEFINED	0
#define ES_DB_SWAP_CONTENT_CLEAR_TO_BACKGROUND  1
#define ES_DB_SWAP_CONTENT_UNCHANGED	2
#define ES_DB_SWAP_CONTENT_FRONTBUFFER	3

#define HP_RENDER_TO_BACK_BUFFER        0
#define HP_RENDER_TO_FRONT_BUFFER       1

/* For compatibility */
#define ES_ESCAPE_DBLBUFFER \
		PEXEncodeEscape(ES_VENDOR_ID, ESEscapeDoubleBufferID)
#define ES_ESCAPE_SWAPBUFFER \
		PEXEncodeEscape(ES_VENDOR_ID, ESEscapeSwapBufferID)
#define ES_ESCAPE_SWAPBUFFERCONTENT \
		PEXEncodeEscape(ES_VENDOR_ID, ESEscapeSwapBufferContentID)
#define HP_ESCAPE_DFRONT \
		PEXEncodeEscape(HP_VENDOR_ID, HPEscapeDFrontID)

/* Data structures for E&S and HP double buffering, used to call PEXEscape. */

typedef struct   /* Request used for turning double buffering on and off. */
{
   Drawable        drawable;
   unsigned int   bufferMode;
} esEscapeDblBuffer;

typedef struct   /* Request used to swap the buffers. */
{
   Drawable        drawable;
} esEscapeSwapBuffer;

typedef struct   /* Request used to inquire content of swapped buffer. */
{
   Drawable        drawable;
} esEscapeSwapBufferContent;

typedef struct   /* Reply describing the content of the swapped buffer. */
{
   unsigned int   content;
} esEscapeSwapBufferContentReply;

typedef struct   /* Request used for setting front buffer as rendering buffer.*/
{
   Drawable        drawable;
   int             render_to_front_buffer;
} hpEscapeSetRenderingBuffer;
