/*
 *  File Name: draw_diagram5.c
 * 
 *  Description: Draw diagrams in Chapter 5
 *
 *  Author: Paula Womack
 *
 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/PEX5/PEXlib.h>
#include "util_protos.h"

#define	TITLE 		"Chapter5 diagrams"

#define N_LINE_VERT 2
static PEXCoord lines[N_LINE_VERT] =
{
 {0.1,  0.2,  0.0},
 {0.9,  0.2,  0.0},
};

#define N_MARKER_VERT 5
static PEXCoord markers[N_MARKER_VERT] =
{
 {0.1,  0.1,  0.0},
 {0.3,  0.1,  0.0},
 {0.5,  0.1,  0.0},
 {0.7,  0.1,  0.0},
 {0.9,  0.1,  0.0},
};

#define N_CURVE_VERT 7
static PEXCoord control_pts[N_CURVE_VERT] =
{
 {0.05,  0.07,  0.0},
 {0.2,   0.22,  0.0},
 {0.35,  0.07,  0.0},
 {0.5,   0.22,  0.0},
 {0.65,  0.07,  0.0},
 {0.8,   0.22,  0.0},
 {0.95,  0.07,  0.0},
};

static PEXColorRGB  black_color = {0.0, 0.0, 0.0};

int translate_pts( int n, PEXCoord *pts, float xtrans, float ytrans, float ztrans)
{
    int      i;
    PEXCoord *nextPt = pts;

    for  (i= 0; i < n; i++, nextPt++) {
	nextPt->x += xtrans;
	nextPt->y += ytrans;
	nextPt->z += ztrans;
    }
}

/*
 * Show different line styles
 */
int draw_diagram51( Display *dpy, Window win, PEXRenderer rd )
{
    PEXCoord pts[N_LINE_VERT];
    int i;

    for (i= 0; i < N_LINE_VERT; i++)
       pts[i] = lines[i];

    PEXBeginRendering( dpy, win, rd);
	PEXSetLineWidth( dpy, rd, PEXOCRender, 2.0);
	PEXSetLineColor( dpy, rd, PEXOCRender, PEXColorTypeRGB, 
            (PEXColor *)&black_color);

	PEXSetLineType( dpy, rd, PEXOCRender, PEXLineTypeSolid);
	PEXPolyline( dpy, rd, PEXOCRender, N_LINE_VERT, pts);

        translate_pts( N_LINE_VERT, pts, 0.0, .2, 0.0);
	PEXSetLineType( dpy, rd, PEXOCRender, PEXLineTypeDashed);
	PEXPolyline( dpy, rd, PEXOCRender, N_LINE_VERT, pts);

        translate_pts( N_LINE_VERT, pts, 0.0, .2, 0.0);
	PEXSetLineType( dpy, rd, PEXOCRender, PEXLineTypeDotted);
	PEXPolyline( dpy, rd, PEXOCRender, N_LINE_VERT, pts);

        translate_pts( N_LINE_VERT, pts, 0.0, .2, 0.0);
	PEXSetLineType( dpy, rd, PEXOCRender, PEXLineTypeDashDot);
	PEXPolyline( dpy, rd, PEXOCRender, N_LINE_VERT, pts);
    PEXEndRendering( dpy, rd, True);
}

/*
 * Show different marker types
 */
int draw_diagram53( Display *dpy, Window win, PEXRenderer rd )
{
    PEXCoord pts[N_MARKER_VERT];
    int i;

    for (i= 0; i < N_MARKER_VERT; i++)
       pts[i] = markers[i];

    PEXBeginRendering( dpy, win, rd);
	PEXSetMarkerColor( dpy, rd, PEXOCRender, PEXColorTypeRGB, 
            (PEXColor *)&black_color);

	PEXSetMarkerType( dpy, rd, PEXOCRender, PEXMarkerDot);
	PEXMarkers( dpy, rd, PEXOCRender, N_MARKER_VERT, pts);

        translate_pts( N_MARKER_VERT, pts, 0.0, .2, 0.0);
	PEXSetMarkerType( dpy, rd, PEXOCRender, PEXMarkerCross);
	PEXMarkers( dpy, rd, PEXOCRender, N_MARKER_VERT, pts);

        translate_pts( N_MARKER_VERT, pts, 0.0, .2, 0.0);
	PEXSetMarkerType( dpy, rd, PEXOCRender, PEXMarkerAsterisk);
	PEXMarkers( dpy, rd, PEXOCRender, N_MARKER_VERT, pts);

        translate_pts( N_MARKER_VERT, pts, 0.0, .2, 0.0);
	PEXSetMarkerType( dpy, rd, PEXOCRender, PEXMarkerCircle);
	PEXMarkers( dpy, rd, PEXOCRender, N_MARKER_VERT, pts);

        translate_pts( N_MARKER_VERT, pts, 0.0, .2, 0.0);
	PEXSetMarkerType( dpy, rd, PEXOCRender, PEXMarkerX);
	PEXMarkers( dpy, rd, PEXOCRender, N_MARKER_VERT, pts);
    PEXEndRendering( dpy, rd, True);
}

/*
 * Show affect of varying order for b-spline curve 
 */
int draw_diagram56( Display *dpy, Window win, PEXRenderer rd )
{

    PEXCoord pts[N_CURVE_VERT];
    float knots[2*N_CURVE_VERT];
    double tmin, tmax;
    int order, i;
    PEXArrayOfCoord curve_data;

    for (i= 0; i < N_CURVE_VERT; i++)
       pts[i] = control_pts[i];
    curve_data.point = pts;

    /* Just set  tmin and tmax to min and max knot values */
    tmin = 0.0;
    tmax = 1.0;

    PEXBeginRendering( dpy, win, rd);
	PEXSetMarkerColor( dpy, rd, PEXOCRender, PEXColorTypeRGB, 
            (PEXColor *)&black_color);
	PEXSetMarkerType( dpy, rd, PEXOCRender, PEXMarkerDot);
	PEXSetLineWidth( dpy, rd, PEXOCRender, 2.0);
	PEXSetLineColor( dpy, rd, PEXOCRender, PEXColorTypeRGB, 
            (PEXColor *)&black_color);

	/* 
	 * 7 control points and order 4 requires 11 knots
	 * The first and last knot values must be repeated 4 times
	 */
	order = 4;
	knots[0] = knots[1] = knots[2] = knots[3] = 0.0;
	knots[4] = .25;
	knots[5] = .50;
	knots[6] = .75;
	knots[7] = knots[8] = knots[9] = knots[10] = 1.0;
 
        /* Draw curve with order 4; use markers to show control points */
        PEXMarkers( dpy, rd, PEXOCRender, N_CURVE_VERT, pts);
        PEXNURBCurve( dpy, rd, PEXOCRender, PEXNonRational, order, knots, 
		N_CURVE_VERT, curve_data, tmin, tmax);

	/* 
	 * 7 control points and order 5 requires 12 knots
	 * The first and last knot values must be repeated 4 times
	 */
	order = 5;
	knots[0] = knots[1] = knots[2] = knots[3] = knots[4] = 0.0;
	knots[5] = .33;
	knots[6] = .66;
	knots[7] = knots[8] = knots[9] = knots[10] = knots[11] = 1.0;
 
        /* Draw curve with order 5; use markers to show control points */
        translate_pts( N_CURVE_VERT, pts, 0.0, .35, 0.0);
        PEXMarkers( dpy, rd, PEXOCRender, N_CURVE_VERT, pts);
        PEXNURBCurve( dpy, rd, PEXOCRender, PEXNonRational, order, knots, 
		N_CURVE_VERT, curve_data, tmin, tmax);

	/* 
	 * 7 control points and order 6 requires 13 knots
	 * The first and last knot values must be repeated 4 times
	 */
	order = 6;
	knots[0] = knots[1] = knots[2] = knots[3] = knots[4] = knots[5] = 0.0;
	knots[6] = .5;
	knots[7] = knots[8] = knots[9] = knots[10] = knots[11] = knots[12] = 1.0;
 
        /* Draw curve with order 6; use markers to show control points */
        translate_pts( N_CURVE_VERT, pts, 0.0, .35, 0.0);
        PEXMarkers( dpy, rd, PEXOCRender, N_CURVE_VERT, pts);
        PEXNURBCurve( dpy, rd, PEXOCRender, PEXNonRational, order, knots, 
		N_CURVE_VERT, curve_data, tmin, tmax);

    PEXEndRendering( dpy, rd, True);
}

/*
 * Show affect of changing knots
 */
int draw_diagram57( Display *dpy, Window win, PEXRenderer rd )
{
    PEXCoord pts[N_CURVE_VERT];
    float knots[2*N_CURVE_VERT];
    double tmin, tmax;
    int order, i;
    PEXArrayOfCoord curve_data;

    for (i= 0; i < N_CURVE_VERT; i++)
       pts[i] = control_pts[i];
    curve_data.point = pts;

    /* Just set  tmin and tmax to min and max knot values */
    tmin = 0.0;
    tmax = 1.0;

    PEXBeginRendering( dpy, win, rd);
	PEXSetMarkerColor( dpy, rd, PEXOCRender, PEXColorTypeRGB, 
            (PEXColor *)&black_color);
	PEXSetMarkerType( dpy, rd, PEXOCRender, PEXMarkerDot);
	PEXSetLineWidth( dpy, rd, PEXOCRender, 2.0);
	PEXSetLineColor( dpy, rd, PEXOCRender, PEXColorTypeRGB, 
            (PEXColor *)&black_color);

	/* 
	 * Use uniform knot list
	 */
	order = 4;
	knots[0] = knots[1] = knots[2] = knots[3] = 0.0;
	knots[4] = .25;
	knots[5] = .50;
	knots[6] = .75;
	knots[7] = knots[8] = knots[9] = knots[10] = 1.0;
 
        /* Draw curve with order 4; use markers to show control points */
        translate_pts( N_CURVE_VERT, pts, 0.0, .1, 0.0);
        PEXMarkers( dpy, rd, PEXOCRender, N_CURVE_VERT, pts);
        PEXNURBCurve( dpy, rd, PEXOCRender, PEXNonRational, order, knots, 
		N_CURVE_VERT, curve_data, tmin, tmax);

	/* 
 	 * Repeat knots to create corners
	 */
	order = 4;
	knots[0] = knots[1] = knots[2] = knots[3] = 0.0;
	knots[4] = .50;
	knots[5] = .50;
	knots[6] = .50;
	knots[7] = knots[8] = knots[9] = knots[10] = 1.0;
 
        /* Draw curve with order 5; use markers to show control points */
        translate_pts( N_CURVE_VERT, pts, 0.0, .4, 0.0);
        PEXMarkers( dpy, rd, PEXOCRender, N_CURVE_VERT, pts);
        PEXNURBCurve( dpy, rd, PEXOCRender, PEXNonRational, order, knots, 
		N_CURVE_VERT, curve_data, tmin, tmax);
    PEXEndRendering( dpy, rd, True);
}

int main(int argc, char *argv[])
{
   Display			*dpy;    /* connection information */
   Window			win;     /* window to draw in */
   unsigned int			width, height;
   XEvent	    		event;   /* event record returned by X */
   XVisualInfo 			vis_info; /* color information */
   XStandardColormap 		cmap_info;
   PEXColorApproxEntry         	approx_entry;
   PEXRenderer			rd;      /* PEX renderer resource */
   PEXRendererAttributes	rd_attr;
   unsigned long		rd_flags;
   PEXExtensionInfo 		*ext_info;/* info about PEX extension */
   int				done;   /*indicates when program is done*/
   char 			key_pressed;
   int				demo;

   /* Open a connection to the PEX server */
   dpy = open_pex_connection( &ext_info);
   if ( dpy == NULL ) exit( 1);

   /* Find best visual and colormap. If we fail, just exit */
   if ( !setup_colors( dpy, &vis_info, &cmap_info, &approx_entry) )
      exit( 1);

   /* Create a window */
   win = create_window( dpy, TITLE, WhitePixel( dpy, DefaultScreen( dpy)),
           &vis_info, &cmap_info, &width, &height);

   /* Ask for mouse press and window exposure events */
   XSelectInput( dpy, win, KeyPressMask | ButtonPressMask | ExposureMask);

   /* If PEX version 5.1 or later then let PEX clear the window to 
    * white before rendering */
   rd_flags = 0;
   if ( ext_info->minor_version > 0 ) {
      rd_attr.clear_image = True;
      rd_attr.background_color.type = PEXColorTypeRGB;
      rd_attr.background_color.value.rgb.red = 1.0;
      rd_attr.background_color.value.rgb.green = 1.0;
      rd_attr.background_color.value.rgb.blue = 1.0;
      rd_flags |= PEXRAClearImage | PEXRABackgroundColor;
   }

   /* Create a PEX renderer with the attributes specified above */
   rd = create_renderer( dpy, win, &approx_entry, rd_flags, &rd_attr);

   /* Map the window */
   XMapWindow( dpy, win);

   /* Event loop */
   done = False;
   demo = 1;
   while ( !done ) {
      XNextEvent( dpy, &event);
      switch ( event.type ) {
         /* 
          * Whenever we get an expose event we must:
          *    Get rid of all other expose events on the queue
          *    Clear the window if we are talking to a PEX 5.0 server
          *    Redraw the first diagram
          */
	 case Expose: 
            while ( XCheckTypedWindowEvent( dpy, win, Expose, &event) );
            if ( ext_info->minor_version < 1 )
               XClearWindow( dpy, win);
            draw_diagram51( dpy, win, rd);
            break;

	/*
         * Show the next diagram whenever 'n' or 'N' is typed
         */
         case KeyPress:
	    XLookupString( (XKeyEvent *)&event, &key_pressed, 1, NULL, NULL);
	    if (key_pressed == 'n' || key_pressed == 'N') {
                demo++; 
                if (demo > 4) demo = 1;

                if ( ext_info->minor_version < 1 )
                    XClearWindow( dpy, win);

		if (demo == 1)
		    draw_diagram51( dpy, win, rd);
		else if (demo == 2)
		    draw_diagram53( dpy, win, rd);
	  	else if (demo == 3)
		    draw_diagram56( dpy, win, rd);
                else  /* demo = 4 */
		    draw_diagram57( dpy, win, rd);
	    }

	    break;

         /* Quit when mouse button is pressed */
	 case ButtonPress:
	    done = True;
	    break;
      }
   }

   /* Termination */
   PEXFreeRenderer( dpy, rd);
   XDestroyWindow( dpy, win);
   XCloseDisplay( dpy);
   exit( 0);
}
