/*
 *  File Name: draw_diagram6.c
 *
 *  Description: Draw diagrams from Chapter 6
 * 
 *  Author: Paula Womack
 *
 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/PEX5/PEXlib.h>
#include "util_protos.h"

#define	TITLE 		"Chapter6 diagrams"

#define N_LINE_VERT 2
static PEXCoord lines[N_LINE_VERT] =
{
 {0.1,  0.2,  0.0},
 {0.9,  0.2,  0.0},
};

#define N_MARKER_VERT 5
static PEXCoord markers[N_MARKER_VERT] =
{
 {0.1,  0.1,  0.0},
 {0.3,  0.1,  0.0},
 {0.5,  0.1,  0.0},
 {0.7,  0.1,  0.0},
 {0.9,  0.1,  0.0},
};

#define N_CURVE_VERT 7
static PEXCoord control_pts[N_CURVE_VERT] =
{
 {0.05,  0.07,  0.0},
 {0.2,   0.22,  0.0},
 {0.35,  0.07,  0.0},
 {0.5,   0.22,  0.0},
 {0.65,  0.07,  0.0},
 {0.8,   0.22,  0.0},
 {0.95,  0.07,  0.0},
};

static PEXColorRGB  black_color = {0.0, 0.0, 0.0};

/*
 * Show text paths
 */
int draw_diagram62( Display *dpy, Window win, PEXRenderer rd )
{
    PEXCoord center, position;
    PEXVector x_dir;
    PEXVector y_dir;

    center.x = center.y = 0.5; center.z = 0.0;
    x_dir.x = 1.0; x_dir.y = 0.0; x_dir.z = 0.0;
    y_dir.x = 0.0; y_dir.y = 1.0; y_dir.z = 0.0;

    PEXBeginRendering( dpy, win, rd);
	PEXSetMarkerColor( dpy, rd, PEXOCRender, PEXColorTypeRGB, 
            (PEXColor *)&black_color);
        PEXSetMarkerType( dpy, rd, PEXOCRender, PEXMarkerDot);

	PEXSetTextColor( dpy, rd, PEXOCRender, PEXColorTypeRGB, 
            (PEXColor *)&black_color);
        PEXSetCharHeight( dpy, rd, PEXOCRender, 0.03);
        PEXSetTextPrecision( dpy, rd, PEXOCRender, PEXStrokePrecision);

        position = center; position.x += .05;
        PEXSetTextPath( dpy, rd, PEXOCRender, PEXPathRight);
        PEXMarkers( dpy, rd, PEXOCRender, 1, &position);
        PEXText( dpy, rd, PEXOCRender, &position, &x_dir, &y_dir, 
		strlen("PATH RIGHT"), "PATH RIGHT");

        position = center; position.x -= .05;
        PEXSetTextPath( dpy, rd, PEXOCRender, PEXPathLeft);
        PEXMarkers( dpy, rd, PEXOCRender, 1, &position);
        PEXText( dpy, rd, PEXOCRender, &position, &x_dir, &y_dir, 
		strlen("PATH LEFT"), "PATH LEFT");

        position = center; position.y += .05;
        PEXSetTextPath( dpy, rd, PEXOCRender, PEXPathUp);
        PEXMarkers( dpy, rd, PEXOCRender, 1, &position);
        PEXText( dpy, rd, PEXOCRender, &position, &x_dir, &y_dir, 
		strlen("PATH UP"), "PATH UP");

        position = center; position.y -= .05;
        PEXSetTextPath( dpy, rd, PEXOCRender, PEXPathDown);
        PEXMarkers( dpy, rd, PEXOCRender, 1, &position);
        PEXText( dpy, rd, PEXOCRender, &position, &x_dir, &y_dir, 
		strlen("PATH DOWN"), "PATH DOWN");
    PEXEndRendering( dpy, rd, True);
}

/*
 * Show text path with different up vectors
 */
int draw_diagram63( Display *dpy, Window win, PEXRenderer rd )
{
    PEXCoord position, center;
    PEXVector x_dir;
    PEXVector y_dir;
    PEXVector2D char_up;

    center.x = center.y = 0.5; center.z = 0.0;
    x_dir.x = 1.0; x_dir.y = 0.0; x_dir.z = 0.0;
    y_dir.x = 0.0; y_dir.y = 1.0; y_dir.z = 0.0;
    char_up.x = -1.0; char_up.y = 1.0;

    PEXBeginRendering( dpy, win, rd);
	PEXSetMarkerColor( dpy, rd, PEXOCRender, PEXColorTypeRGB,  
           (PEXColor *)&black_color);
	PEXSetMarkerType( dpy, rd, PEXOCRender, PEXMarkerDot);

	PEXSetTextColor( dpy, rd, PEXOCRender, PEXColorTypeRGB, 
            (PEXColor *)&black_color);
        PEXSetCharHeight( dpy, rd, PEXOCRender, 0.03);
	PEXSetCharUpVector( dpy, rd, PEXOCRender, &char_up);
        PEXSetTextPrecision( dpy, rd, PEXOCRender, PEXStrokePrecision);

        position = center; 
        position.x += .035; position.y += .035;
        PEXSetTextPath( dpy, rd, PEXOCRender, PEXPathRight);
        PEXMarkers( dpy, rd, PEXOCRender, 1, &position);
        PEXText( dpy, rd, PEXOCRender, &position, &x_dir, &y_dir, 
		strlen("PATH RIGHT"), "PATH RIGHT");

        position = center; 
        position.x -= .035; position.y -= .035;
        PEXSetTextPath( dpy, rd, PEXOCRender, PEXPathLeft);
        PEXMarkers( dpy, rd, PEXOCRender, 1, &position);
        PEXText( dpy, rd, PEXOCRender, &position, &x_dir, &y_dir, 
		strlen("PATH LEFT"), "PATH LEFT");

        position = center; 
        position.x -= .035; position.y += .035;
        PEXSetTextPath( dpy, rd, PEXOCRender, PEXPathUp);
        PEXMarkers( dpy, rd, PEXOCRender, 1, &position);
        PEXText( dpy, rd, PEXOCRender, &position, &x_dir, &y_dir, 
		strlen("PATH UP"), "PATH UP");

        position = center;
        position.x += .035; position.y -= .035;
        PEXSetTextPath( dpy, rd, PEXOCRender, PEXPathDown);
        PEXMarkers( dpy, rd, PEXOCRender, 1, &position);
        PEXText( dpy, rd, PEXOCRender, &position, &x_dir, &y_dir, 
		strlen("PATH DOWN"), "PATH DOWN");
    PEXEndRendering( dpy, rd, True);
}

/*
 * Show affect of character expansion
 */
int draw_diagram65( Display *dpy, Window win, PEXRenderer rd )
{
    PEXCoord position;
    PEXVector x_dir;
    PEXVector y_dir;

    position.x = 0.05; position.y = 0.2; position.z = 0.0;
    x_dir.x = 1.0; x_dir.y = 0.0; x_dir.z = 0.0;
    y_dir.x = 0.0; y_dir.y = 1.0; y_dir.z = 0.0;

    PEXBeginRendering( dpy, win, rd);
	PEXSetTextColor( dpy, rd, PEXOCRender, PEXColorTypeRGB, 
            (PEXColor *)&black_color);
        PEXSetCharHeight( dpy, rd, PEXOCRender, 0.03);
        PEXSetTextPrecision( dpy, rd, PEXOCRender, PEXStrokePrecision);

        PEXText( dpy, rd, PEXOCRender, &position, &x_dir, &y_dir, 
	    strlen("DEFAULT EXPANSION FACTOR"), "DEFAULT EXPANSION FACTOR");

        position.y += .2;
	PEXSetCharExpansion( dpy, rd, PEXOCRender, 0.7);
        PEXText( dpy, rd, PEXOCRender, &position, &x_dir, &y_dir, 
	    strlen("DECREASE EXPANSION FACTOR"), "DECREASE EXPANSION FACTOR");

        position.y += .2;
	PEXSetCharExpansion( dpy, rd, PEXOCRender, 1.8);
        PEXText( dpy, rd, PEXOCRender, &position, &x_dir, &y_dir, 
	    strlen("INCREASE EXPANSION FACTOR"), "INCREASE EXPANSION FACTOR");

    PEXEndRendering( dpy, rd, True);

}


int main(int argc, char *argv[])
{
   Display			*dpy;    /* connection information */
   Window			win;     /* window to draw in */
   unsigned int			width, height;
   XEvent	    		event;   /* event record returned by X */
   XVisualInfo 			vis_info; /* color information */
   XStandardColormap 		cmap_info;
   PEXColorApproxEntry         	approx_entry;
   PEXRenderer			rd;      /* PEX renderer resource */
   PEXRendererAttributes	rd_attr;
   unsigned long		rd_flags;
   PEXExtensionInfo 		*ext_info;/* info about PEX extension */
   int				done;   /*indicates when program is done*/
   char 			key_pressed;
   int				demo;

   /* Open a connection to the PEX server */
   dpy = open_pex_connection( &ext_info);
   if ( dpy == NULL ) exit( 1);

   /* Find best visual and colormap. If we fail, just exit */
   if ( !setup_colors( dpy, &vis_info, &cmap_info, &approx_entry) )
      exit( 1);

   /* Create a window */
   win = create_window( dpy, TITLE, WhitePixel( dpy, DefaultScreen( dpy)),
           &vis_info, &cmap_info, &width, &height);

   /* Ask for mouse press and window exposure events */
   XSelectInput( dpy, win, KeyPressMask | ButtonPressMask | ExposureMask);

   /* If PEX version 5.1 or later then let PEX clear the window to 
    * white before rendering */
   rd_flags = 0;
   if ( ext_info->minor_version > 0 ) {
      rd_attr.clear_image = True;
      rd_attr.background_color.type = PEXColorTypeRGB;
      rd_attr.background_color.value.rgb.red = 1.0;
      rd_attr.background_color.value.rgb.green = 1.0;
      rd_attr.background_color.value.rgb.blue = 1.0;
      rd_flags |= PEXRAClearImage | PEXRABackgroundColor;
   }

   /* Create a PEX renderer with the attributes specified above */
   rd = create_renderer( dpy, win, &approx_entry, rd_flags, &rd_attr);

   /* Map the window */
   XMapWindow( dpy, win);

   /* Event loop */
   done = False;
   demo = 1;
   while ( !done ) {
      XNextEvent( dpy, &event);
      switch ( event.type ) {
         /* 
          * Whenever we get an expose event we must:
          *    Get rid of all other expose events on the queue
          *    Clear the window if we are talking to a PEX 5.0 server
          *    Redraw the first diagram
          */
	 case Expose: 
            while ( XCheckTypedWindowEvent( dpy, win, Expose, &event) );
            if ( ext_info->minor_version < 1 )
               XClearWindow( dpy, win);
            draw_diagram62( dpy, win, rd);
            break;

	/*
         * Show the next diagram whenever 'n' or 'N' is typed
         */
         case KeyPress:
	    XLookupString( (XKeyEvent *)&event, &key_pressed, 1, NULL, NULL);
	    if (key_pressed == 'n' || key_pressed == 'N') {
                demo++; 
                if (demo > 3) demo = 1;

                if ( ext_info->minor_version < 1 )
                    XClearWindow( dpy, win);

		if (demo == 1)
		    draw_diagram62( dpy, win, rd);
	  	else if (demo == 2)
		    draw_diagram63( dpy, win, rd);
                else  /* demo = 3 */
		    draw_diagram65( dpy, win, rd);
	    }

	    break;

         /* Quit when mouse button is pressed */
	 case ButtonPress:
	    done = True;
	    break;
      }
   }

   /* Termination */
   PEXFreeRenderer( dpy, rd);
   XDestroyWindow( dpy, win);
   XCloseDisplay( dpy);
   exit( 0);
}
