/*
 *  File Name: simple_lines.c
 * 
 *  Author: Paula Womack
 *
 *  Revision: 1.0
 *
 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/PEX5/PEXlib.h>
#include "util_protos.h"

#define	TITLE 		"PEX Lines"

#define N_LINE 5
static PEXCoord diagonal1[N_LINE] =
{
 {0.1,   0.1,  0.0},
 {0.2,   0.1,  0.0},
 {0.9,   0.9,  0.0},
 {0.8,   0.9,  0.0},
 {0.1,   0.1,  0.0},
};
static PEXCoord diagonal2[N_LINE] =
{
 {0.1,   0.9,  0.0},
 {0.2,   0.9,  0.0},
 {0.9,   0.1,  0.0},
 {0.8,   0.1,  0.0},
 {0.1,   0.9,  0.0},
};
static PEXColorRGB  red_color = {1.0, 0.0, 0.0};

int draw_lines(dpy, win, rd)
Display *dpy; 
Window win; 
PEXRenderer rd;
{
    PEXBeginRendering (dpy, win, rd);
	PEXSetLineWidth (dpy, rd, PEXOCRender, 2.0);
        PEXSetLineColor(dpy, rd, PEXOCRender, PEXColorTypeRGB, 
	    (PEXColor *)&red_color);
        PEXPolyline(dpy, rd, PEXOCRender, N_LINE, diagonal1);
        PEXPolyline(dpy, rd, PEXOCRender, N_LINE, diagonal2);
    PEXEndRendering (dpy, rd, True);
}


int main(argc, argv)
int argc; 
char *argv[];
{
    Display			*dpy;    /* connection information */
    Window			win;     /* window to draw in */
    unsigned int		width, height;
    XEvent	    		event;   /* event record returned by X */
    XVisualInfo                 vis_info; /* color information */
    XStandardColormap           cmap_info;
    PEXColorApproxEntry         approx_entry;
    PEXRenderer			rd;      /* PEX renderer resource */
    PEXRendererAttributes	rd_attr;
    unsigned long		rd_flags;
    PEXExtensionInfo 		*ext_info;/* info about PEX extension */
    int				done;    /* indicates when program is done */

    /* Open a connection to the PEX server */
    dpy = open_pex_connection( &ext_info);
    if ( dpy == NULL ) exit( 1);
       
    /* Find best visual and colormap. If we fail, just exit */
    if ( !setup_colors( dpy, &vis_info, &cmap_info, &approx_entry) )
        exit( 1);

    /* Create a window */
    win = create_window( dpy, TITLE, WhitePixel( dpy, DefaultScreen( dpy)), 
            &vis_info, &cmap_info, &width, &height);

    /* Ask for mouse press and window exposure events */
    XSelectInput( dpy, win, ButtonPressMask | ExposureMask);

    /* If PEX version 5.1 or later then let PEX clear the window */
    rd_flags = 0;
    if ( ext_info->minor_version > 0 ) {
        rd_attr.clear_image = True;
        rd_attr.background_color.type = PEXColorTypeRGB;
        rd_attr.background_color.value.rgb.red = 1.0;
        rd_attr.background_color.value.rgb.green = 1.0;
        rd_attr.background_color.value.rgb.blue = 1.0;
        rd_flags |= PEXRAClearImage | PEXRABackgroundColor;
    }

    /* Create a PEX renderer */
    rd = create_renderer( dpy, win, &approx_entry, rd_flags, &rd_attr);

    /* Map the window */
    XMapWindow( dpy, win);

    /* Event loop */
    done = False;
    while ( !done ) {
	XNextEvent( dpy, &event);
        switch ( event.type ) {

            /* 
             * Whenever we get an expose event we must:
             *    Get rid of all other expose events for this window
             *    Clear the window if we are talking to a PEX 5.0 server
             *    Redraw the lines
             */
	    case Expose: 
                while ( XCheckTypedWindowEvent( dpy, win, Expose, &event) );
    		if ( ext_info->minor_version < 1 )
        	    XClearWindow( dpy, win);
	        draw_lines( dpy, win, rd);
		break;

            /* Quit when mouse button is pressed */
	    case ButtonPress:
		done = True;
		break;
	}
    }

    /* Termination */
    XCloseDisplay( dpy);
    exit( 0);
}
