/*
 *  File Name: get_subsets.c
 * 
 *  Author: Paula Womack
 *
 *  Description: Print out PEX subsets (snippet from Chapter 2)
 *
 *  Revision: 1.0
 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/PEX5/PEXlib.h>

int main(int argc, char *argv[])
{
    Display			*dpy;    /* connection information */
    char			*display_name;
    PEXExtensionInfo 		*ext_info;/* info about PEX extension */
    int				subset;
    char			error_string[PEXErrorStringLength];

    /* Open a connection to the X server */
    if ( !(dpy = XOpenDisplay( "")) ) {
       fprintf( stderr, "Cannot open display %s \n", XDisplayName( ""));
       exit( 1);
    }

    /* Initialize the PEX extension and get the PEX extension information  */
    /* We will need to know the version of PEX later on.                   */
    if ( PEXInitialize( dpy, &ext_info, PEXErrorStringLength, error_string) != 0 ) {
	fprintf( stderr, "Error Initializing PEX: %s\n", error_string);
	exit( 1);
    }

   /*
    * This code fragment appears in Chapter 2. It gets the supported subsets 
    * from the PEX server.
    */
    ext_info = PEXGetExtensionInfo( dpy);
    subset = ext_info->subset_info & 0xff;
    if ( subset == PEXCompleteImplementation )
        printf( "All subsets supported\n");
    else {
        if ( subset & PEXImmediateMode )
            printf( "Immediate mode rendering is supported\n");
        if ( subset & PEXStructureMode )
            printf( "Structure mode rendering is supported\n");
        if ( subset & PEXWorkstationOnly )
            printf( "Workstation mode rendering is supported\n");
    }

    XCloseDisplay( dpy);
    exit( 0);
}
